@@ -80,7 +80,11 @@ struct _GtkSourceGutter
 	GtkTextWindowType     window_type;
 	GtkOrientation        orientation;
 
+	double                pointer_x;
+	double                pointer_y;
+
 	guint                 is_drawing : 1;
+	guint                 pointer_in_gutter : 1;
 };
 
 G_DEFINE_TYPE (GtkSourceGutter, gtk_source_gutter, GTK_TYPE_WIDGET)
@@ -320,10 +324,60 @@ gtk_source_gutter_measure (GtkWidget      *widget,
 }
 
 static void
-gtk_source_gutter_set_property (GObject       *object,
-                                guint          prop_id,
-                                const GValue  *value,
-                                GParamSpec    *pspec)
+gtk_source_gutter_motion_cb (GtkSourceGutter          *gutter,
+			     double                    x,
+			     double                    y,
+			     GtkEventControllerMotion *motion)
+{
+	g_assert (GTK_SOURCE_IS_GUTTER (gutter));
+	g_assert (GTK_IS_EVENT_CONTROLLER_MOTION (motion));
+
+	gutter->pointer_x = x;
+	gutter->pointer_y = y;
+	gutter->pointer_in_gutter = TRUE;
+
+	_gtk_source_gutter_queue_draw (gutter);
+}
+
+static void
+gtk_source_gutter_leave_cb (GtkSourceGutter          *gutter,
+			    GtkEventControllerMotion *motion)
+{
+	g_assert (GTK_SOURCE_IS_GUTTER (gutter));
+	g_assert (GTK_IS_EVENT_CONTROLLER_MOTION (motion));
+
+	gutter->pointer_x = -1;
+	gutter->pointer_y = -1;
+	gutter->pointer_in_gutter = FALSE;
+
+	_gtk_source_gutter_queue_draw (gutter);
+}
+
+static void
+gtk_source_gutter_root (GtkWidget *widget)
+{
+	GtkWidget *parent;
+
+	g_assert (GTK_SOURCE_IS_GUTTER (widget));
+
+	GTK_WIDGET_CLASS (gtk_source_gutter_parent_class)->root (widget);
+
+	parent = gtk_widget_get_parent (widget);
+
+	/* The GtkTextViewChild has "overflow" set to Hidden and we
+	 * want to allow drawing over that.
+	 */
+	if (parent != NULL)
+	{
+		gtk_widget_set_overflow (parent, GTK_OVERFLOW_VISIBLE);
+	}
+}
+
+static void
+gtk_source_gutter_set_property (GObject      *object,
+                                guint         prop_id,
+                                const GValue *value,
+                                GParamSpec   *pspec)
 {
 	GtkSourceGutter *gutter = GTK_SOURCE_GUTTER (object);
 
@@ -394,6 +448,7 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	widget_class->measure = gtk_source_gutter_measure;
 	widget_class->size_allocate = gtk_source_gutter_size_allocate;
 	widget_class->snapshot = gtk_source_gutter_snapshot;
+	widget_class->root = gtk_source_gutter_root;
 
 	/**
 	 * GtkSourceGutter:view:
@@ -457,15 +512,15 @@ gtk_source_gutter_init (GtkSourceGutter *gutter)
 	motion = gtk_event_controller_motion_new ();
 	g_signal_connect_swapped (motion,
 	                          "enter",
-	                          G_CALLBACK (gtk_widget_queue_draw),
+	                          G_CALLBACK (gtk_source_gutter_motion_cb),
 	                          gutter);
 	g_signal_connect_swapped (motion,
 	                          "leave",
-	                          G_CALLBACK (gtk_widget_queue_draw),
+	                          G_CALLBACK (gtk_source_gutter_leave_cb),
 	                          gutter);
 	g_signal_connect_swapped (motion,
 	                          "motion",
-	                          G_CALLBACK (gtk_widget_queue_draw),
+	                          G_CALLBACK (gtk_source_gutter_motion_cb),
 	                          gutter);
 	gtk_widget_add_controller (GTK_WIDGET (gutter), motion);
 }
@@ -698,12 +753,14 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 {
 	GtkSourceGutter *gutter = GTK_SOURCE_GUTTER (widget);
 	GtkTextView *text_view = GTK_TEXT_VIEW (gutter->view);
+	GtkTextBuffer *buffer;
 	const GList *list;
 	GdkRectangle visible_rect;
-	GtkTextIter begin;
-	GtkTextIter end;
+	GtkTextIter begin, end;
+	GtkTextIter cur, sel;
 	gboolean needs_wrap_first = FALSE;
 	gboolean needs_wrap_last = FALSE;
+	int clip_width = 0;
 
 	g_clear_object (&gutter->lines);
 
@@ -712,6 +769,8 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 		return;
 	}
 
+	buffer = gtk_text_view_get_buffer (text_view);
+
 	gtk_text_view_get_visible_rect (text_view, &visible_rect);
 	gtk_text_view_get_iter_at_location (text_view, &begin,
 	                                    visible_rect.x, visible_rect.y);
@@ -719,6 +778,14 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 	                                    visible_rect.x,
 	                                    visible_rect.y + visible_rect.height);
 
+	/* Try to include an extra line on each edge so that situations
+	 * that are dependent on neighboring lines can still include enough
+	 * information to draw correctly. This is useful for situations like
+	 * git where you might need to draw special delete marks.
+	 */
+	gtk_text_iter_backward_line (&begin);
+	gtk_text_iter_forward_line (&end);
+
 	/* The first step is to get line information about all the visible
 	 * lines. We do this up front so that we can do it once to reduce many
 	 * times the renderers need to walk through the buffer contents as that
@@ -731,19 +798,43 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 	                                              needs_wrap_first,
 	                                              needs_wrap_last);
 
-	/* Draw the current-line highlight if necessary */
-	if (gtk_source_view_get_highlight_current_line (gutter->view))
+	/* Get the line under the pointer so we can set "prelit" on it */
+	if (gutter->pointer_in_gutter)
 	{
+		GtkTextIter pointer;
+		GdkRectangle pointer_rect;
+
+		gtk_text_view_get_iter_at_location (text_view,
+		                                    &pointer,
+		                                    0,
+		                                    visible_rect.y + gutter->pointer_y);
+		gtk_text_view_get_iter_location (text_view, &pointer, &pointer_rect);
+		pointer_rect.y -= visible_rect.y;
+
+		if (gutter->pointer_y >= pointer_rect.y &&
+		    gutter->pointer_y <= pointer_rect.y + pointer_rect.height)
+		{
+			guint line = gtk_text_iter_get_line (&pointer);
+			gtk_source_gutter_lines_add_class (gutter->lines, line, "prelit");
+		}
+	}
+
+	/* Draw the current-line highlight if necessary */
+	if (gtk_source_view_get_highlight_current_line (gutter->view) &&
+	    !gtk_text_buffer_get_selection_bounds (buffer, &cur, &sel))
+	{
+		GdkRGBA highlight;
 		guint cursor_line;
 
 		cursor_line = _gtk_source_gutter_lines_get_cursor_line (gutter->lines);
 
 		if (cursor_line >= gtk_source_gutter_lines_get_first (gutter->lines) &&
-		    cursor_line <= gtk_source_gutter_lines_get_last (gutter->lines))
+		    cursor_line <= gtk_source_gutter_lines_get_last (gutter->lines) &&
+		    _gtk_source_view_get_current_line_number_background (gutter->view, &highlight))
 		{
-			GdkRGBA highlight;
-			gint y;
-			gint height;
+			int width = gtk_widget_get_width (widget);
+			int height;
+			int y;
 
 			gtk_source_gutter_lines_get_line_yrange (gutter->lines,
 			                                         cursor_line,
@@ -751,14 +842,9 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 			                                         &y,
 			                                         &height);
 
-			if (_gtk_source_view_get_current_line_number_background (gutter->view, &highlight))
-			{
-				int width = gtk_widget_get_width (widget);
-
-				gtk_snapshot_append_color (snapshot,
-				                           &highlight,
-				                           &GRAPHENE_RECT_INIT (0, y, width, height));
-			}
+			gtk_snapshot_append_color (snapshot,
+			                           &highlight,
+			                           &GRAPHENE_RECT_INIT (0, y, width, height));
 		}
 	}
 
@@ -781,10 +867,18 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 		                                   gutter->lines);
 	}
 
+	clip_width = gtk_widget_get_width (widget);
+
+	/* Allow drawing over the left margin from renderers */
+	if (gutter->window_type == GTK_TEXT_WINDOW_LEFT)
+	{
+		clip_width += gtk_text_view_get_left_margin (text_view);
+	}
+
 	gtk_snapshot_push_clip (snapshot,
 	                        &GRAPHENE_RECT_INIT (0,
 	                                             0,
-	                                             gtk_widget_get_width (widget),
+						     clip_width,
 	                                             gtk_widget_get_height (widget)));
 
 	/* Now let the renderers draw the content for each line. Because
