@@ -23,6 +23,8 @@
 #include <gtk/gtk.h>
 #include <gtksourceview/gtksource.h>
 
+#include "testsuite-gresources.h"
+
 /* If we are running from the source dir (e.g. during make check)
  * we override the path to read from the data dir
  */
@@ -121,6 +123,14 @@ test_guess_language_empty_empty (void)
 	gtk_source_language_manager_guess_language (lm, "", "");
 }
 
+static inline void
+assert_null_language (GtkSourceLanguage *l)
+{
+  if (l != NULL)
+    g_error ("Expected NULL language, got %s",
+             gtk_source_language_get_id (l));
+}
+
 static void
 test_guess_language (void)
 {
@@ -142,16 +152,16 @@ test_guess_language (void)
 	g_test_trap_assert_failed ();
 
 	l = gtk_source_language_manager_guess_language (lm, "foo.abcdef", NULL);
-	g_assert_null (l);
+	assert_null_language (l);
 
 	l = gtk_source_language_manager_guess_language (lm, "foo.abcdef", "");
-	g_assert_null (l);
+	assert_null_language (l);
 
 	l = gtk_source_language_manager_guess_language (lm, NULL, "image/png");
-	g_assert_null (l);
+	assert_null_language (l);
 
 	l = gtk_source_language_manager_guess_language (lm, "", "image/png");
-	g_assert_null (l);
+	assert_null_language (l);
 
 	l = gtk_source_language_manager_guess_language (lm, "foo.c", NULL);
 	g_assert_cmpstr (gtk_source_language_get_id (l), ==, "c");
@@ -185,6 +195,76 @@ test_guess_language (void)
 #endif
 }
 
+static void
+test_resources (void)
+{
+	GtkSourceLanguageManager *lm = gtk_source_language_manager_new ();
+	GtkSourceLanguage *l;
+	GtkSourceBuffer *buffer;
+	const char * search_path[] = { "resource:///language-specs/", NULL, NULL };
+	const char * const *ids;
+	char *dir;
+
+	dir = g_build_filename (TOP_SRCDIR, "data", "language-specs", NULL);
+	search_path[1] = dir;
+
+	g_resources_register (testsuite_get_resource ());
+
+	gtk_source_language_manager_set_search_path (lm, search_path);
+	ids = gtk_source_language_manager_get_language_ids (lm);
+
+	g_assert_true (g_strv_contains (ids, "testsuite"));
+	g_assert_true (g_strv_contains (ids, "testsuite-2"));
+
+	l = gtk_source_language_manager_get_language (lm, "testsuite");
+	g_assert_nonnull (l);
+	g_assert_cmpstr ("testsuite", ==, gtk_source_language_get_id (l));
+
+	buffer = gtk_source_buffer_new (NULL);
+	gtk_source_buffer_set_language (buffer, l);
+
+	g_object_unref (buffer);
+	g_object_unref (lm);
+	g_free (dir);
+}
+
+static void
+on_notify_search_path_cb (GtkSourceLanguageManager *lm,
+                          GParamSpec *pspec,
+                          guint *count)
+{
+  (*count)++;
+}
+
+static void
+test_search_path (void)
+{
+  GtkSourceLanguageManager *lm;
+  static const char * const first[] = { "first", NULL };
+  const char * const *search_path;
+  guint count = 0;
+
+  lm = gtk_source_language_manager_new ();
+  gtk_source_language_manager_set_search_path (lm, first);
+  g_signal_connect (lm, "notify::search-path", G_CALLBACK (on_notify_search_path_cb), &count);
+  gtk_source_language_manager_prepend_search_path (lm, "zero");
+  g_assert_cmpint (count, ==, 1);
+  gtk_source_language_manager_append_search_path (lm, "second");
+  g_assert_cmpint (count, ==, 2);
+  gtk_source_language_manager_append_search_path (lm, "resource:///third");
+  g_assert_cmpint (count, ==, 3);
+  search_path = gtk_source_language_manager_get_search_path (lm);
+
+  g_assert_nonnull (search_path);
+  g_assert_cmpstr (search_path[0], ==, "zero");
+  g_assert_cmpstr (search_path[1], ==, "first");
+  g_assert_cmpstr (search_path[2], ==, "second");
+  g_assert_cmpstr (search_path[3], ==, "resource:///third");
+  g_assert_null (search_path[4]);
+
+  g_assert_finalize_object (lm);
+}
+
 int
 main (int argc, char** argv)
 {
@@ -199,6 +279,8 @@ main (int argc, char** argv)
 	g_test_add_func ("/LanguageManager/guess-language/subprocess/empty_null", test_guess_language_empty_null);
 	g_test_add_func ("/LanguageManager/guess-language/subprocess/null_empty", test_guess_language_null_empty);
 	g_test_add_func ("/LanguageManager/guess-language/subprocess/empty_empty", test_guess_language_empty_empty);
+	g_test_add_func ("/LanguageManager/resources", test_resources);
+	g_test_add_func ("/LanguageManager/search-path", test_search_path);
 
 	return g_test_run();
 }
